@@ -91,7 +91,7 @@ static volatile boolean_t running = false;
  */
 boolean_t can_http() {
         if ((Run.httpd.flags & Httpd_Net || Run.httpd.flags & Httpd_Unix) && (Run.flags & Run_Daemon)) {
-                if (! Engine_hasHostsAllow() && ! Run.httpd.credentials) {
+                if (! Engine_hasHostsAllow() && ! Run.httpd.credentials && ! ((Run.httpd.flags & Httpd_Ssl) && (Run.httpd.flags & Httpd_Net) && Run.httpd.socket.net.ssl.clientpem)) {
                         LogError("%s: monit httpd not started since no connect allowed\n", prog);
                         return false;
 
@@ -137,6 +137,8 @@ void monit_http(Httpd_Action action) {
 
 
 static void *thread_wrapper(void *arg) {
+        sigset_t ns;
+        set_signal_block(&ns, NULL);
         Engine_start();
 #ifdef HAVE_OPENSSL
         Ssl_threadCleanup();
