@@ -341,7 +341,7 @@ static void do_reinit() {
         gc();
 
         if (! parse(Run.files.control)) {
-                LogError("%s daemon died\n", prog);
+                LogError("%s stopped -- configuration file parsing error\n", prog);
                 exit(1);
         }
 
@@ -362,7 +362,7 @@ static void do_reinit() {
         file_init();
 
         if (! file_createPidFile(Run.files.pid)) {
-                LogError("%s daemon died\n", prog);
+                LogError("%s stopped -- cannot create a pid file\n", prog);
                 exit(1);
         }
 
@@ -402,30 +402,25 @@ static void do_action(char **args) {
                    IS(action, "unmonitor") ||
                    IS(action, "restart")) {
                 if (Run.mygroup || service) {
-                        int errors = 0;
-                        boolean_t (*_control_service)(const char *, const char *) = exist_daemon() ? control_service_daemon : control_service_string;
-
+                        List_T services = List_new();
                         if (Run.mygroup) {
                                 for (ServiceGroup_T sg = servicegrouplist; sg; sg = sg->next) {
                                         if (IS(Run.mygroup, sg->name)) {
                                                 for (list_t m = sg->members->head; m; m = m->next) {
                                                         Service_T s = m->e;
-                                                        if (! _control_service(s->name, action))
-                                                                errors++;
+                                                        List_append(services, s->name);
                                                 }
                                                 break;
                                         }
                                 }
                         } else if (IS(service, "all")) {
-                                for (Service_T s = servicelist; s; s = s->next) {
-                                        if (s->visited)
-                                                continue;
-                                        if (! _control_service(s->name, action))
-                                                errors++;
-                                }
+                                for (Service_T s = servicelist; s; s = s->next)
+                                        List_append(services, s->name);
                         } else {
-                                errors = _control_service(service, action) ? 0 : 1;
+                                List_append(services, service);
                         }
+                        int errors = exist_daemon() ? control_service_daemon(services, action) : control_service_string(services, action);
+                        List_free(&services);
                         if (errors)
                                 exit(1);
                 } else {
@@ -819,6 +814,8 @@ static void version() {
  * M/Monit heartbeat thread
  */
 static void *heartbeat(void *args) {
+        sigset_t ns;
+        set_signal_block(&ns, NULL);
         LogInfo("M/Monit heartbeat started\n");
         LOCK(heartbeatMutex)
         {
