@@ -61,7 +61,6 @@
 #include <mach/mach_host.h>
 #endif
 
-
 #include "monit.h"
 #include "process.h"
 #include "process_sysdep.h"
@@ -85,6 +84,55 @@ static int  pagesize_kbyte;
 static long total_old    = 0;
 static long cpu_user_old = 0;
 static long cpu_syst_old = 0;
+static boolean_t isSipEnabled = true;
+
+
+/* ----------------------- OS X >= 10.11 System Integrity Protection Check */
+
+/* Check if OS X 10.11 System Integrity Protection (SIP) is enabled. The idea
+ here is to collect all pids via processor_set_tasks() and if we have pid = 1 in
+ the list, then SIP is _not_ enabled, otherwise we assume it is. If SIP
+ is enabled, we are not allowed to call task_for_pid. The alternative is to 
+ call the private API csr_check(CSR_ALLOW_TASK_FOR_PID) but this API is only 
+ supported back to 10.10 AFAIK while we need to support systems all the way
+ back to 10.6 and test the feature at runtime not at build time (via ifdefs) 
+ @return true if enabled otherwise false */
+static boolean_t _isSipEnabled() {
+        isSipEnabled = true;
+        host_t myhost = mach_host_self();
+        mach_port_t psDefault;
+        kern_return_t status = processor_set_default(myhost, &psDefault);
+        mach_port_t psDefaultCtrl;
+        status = host_processor_set_priv(myhost, psDefault, &psDefaultCtrl);
+        if (status != KERN_SUCCESS) {
+                // Will fail if we are not running as root in which case task_for_pid will fail anyway
+                DEBUG("host_processor_set_priv failed -- %s\n", mach_error_string(status));
+                return isSipEnabled;
+        }
+        task_array_t tasks;
+        mach_msg_type_number_t nTasks;
+        status = processor_set_tasks(psDefaultCtrl, &tasks, &nTasks);
+        if (status != KERN_SUCCESS) {
+                DEBUG("processor_set_tasks failed with error -- %s\n", mach_error_string(status));
+                return isSipEnabled;
+        }
+        for (int i = 0; i < nTasks; i++) {
+                int pid;
+                pid_for_task(tasks[i], &pid);
+                if (pid == 1) {
+                        isSipEnabled = false;
+                }
+                mach_port_deallocate(mach_task_self(), tasks[i]);
+        }
+        status= vm_deallocate(mach_task_self(), (vm_address_t)tasks, nTasks * sizeof(task_t));
+        status = mach_port_deallocate(mach_task_self(), psDefaultCtrl);
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_10_11
+        if (isSipEnabled) {
+                DEBUG("System Integrity Protection is enabled and Monit cannot check process memory or CPU usage\n");
+        }
+#endif
+        return isSipEnabled;
+}
 
 
 /* ------------------------------------------------------------------ Public */
@@ -130,6 +178,8 @@ boolean_t init_process_info_sysdep(void) {
         }
         pagesize_kbyte /= 1024;
 
+        isSipEnabled = _isSipEnabled();
+
         return true;
 }
 
@@ -228,42 +278,49 @@ int initprocesstree_sysdep(ProcessTree_T **reference) {
                         pt[i].zombie = true;
                 pt[i].time = get_float_time();
 
-                if (task_for_pid(mytask, pt[i].pid, &task) == KERN_SUCCESS) {
-                        mach_msg_type_number_t   count;
-                        task_basic_info_data_t   taskinfo;
-                        thread_array_t           threadtable;
-                        unsigned int             threadtable_size;
-                        thread_basic_info_t      threadinfo;
-                        thread_basic_info_data_t threadinfo_data;
+                /* Issue #266: As of OS X 10.11 a new System Integrity Protection policy
+                 (SIP) is in use which deny usage of task_for_pid, i.e. we cannot get
+                 process info and to continue here would be useless. SIP is enabled by
+                 default on 10.11. If we are running as non-root 'isSipEnabled' is also
+                 set to true, as we are not allowed to call task_for_pid then neither */
+                if (! isSipEnabled) {
+                        if (task_for_pid(mytask, pt[i].pid, &task) == KERN_SUCCESS) {
+                                mach_msg_type_number_t   count;
+                                task_basic_info_data_t   taskinfo;
+                                thread_array_t           threadtable;
+                                unsigned int             threadtable_size;
+                                thread_basic_info_t      threadinfo;
+                                thread_basic_info_data_t threadinfo_data;
 
-                        count = TASK_BASIC_INFO_COUNT;
-                        if (task_info(task, TASK_BASIC_INFO, (task_info_t)&taskinfo, &count) == KERN_SUCCESS) {
-                                pt[i].mem_kbyte   = (unsigned long)(taskinfo.resident_size / 1024);
-                                pt[i].cputime     = (long)((taskinfo.user_time.seconds + taskinfo.system_time.seconds) * 10 + (taskinfo.user_time.microseconds + taskinfo.system_time.microseconds) / 100000);
-                                pt[i].cpu_percent = 0;
-                        }
-                        if (task_threads(task, &threadtable, &threadtable_size) == KERN_SUCCESS) {
-                                threadinfo = &threadinfo_data;
-                                for (int j = 0; j < threadtable_size; j++) {
-                                        count = THREAD_BASIC_INFO_COUNT;
-                                        if (thread_info(threadtable[j], THREAD_BASIC_INFO, (thread_info_t)threadinfo, &count) == KERN_SUCCESS) {
-                                                if ((threadinfo->flags & TH_FLAGS_IDLE) == 0) {
-                                                        pt[i].cputime += (long)((threadinfo->user_time.seconds + threadinfo->system_time.seconds) * 10 + (threadinfo->user_time.microseconds + threadinfo->system_time.microseconds) / 100000);
-                                                        pt[i].cpu_percent = 0;
-                                                }
-                                        }
-                                        mach_port_deallocate(mytask, threadtable[j]);
+                                count = TASK_BASIC_INFO_COUNT;
+                                if (task_info(task, TASK_BASIC_INFO, (task_info_t)&taskinfo, &count) == KERN_SUCCESS) {
+                                        pt[i].mem_kbyte   = (unsigned long)(taskinfo.resident_size / 1024);
+                                        pt[i].cputime     = (long)((taskinfo.user_time.seconds + taskinfo.system_time.seconds) * 10 + (taskinfo.user_time.microseconds + taskinfo.system_time.microseconds) / 100000);
+                                        pt[i].cpu_percent = 0;
                                 }
-                                vm_deallocate(mytask, (vm_address_t)threadtable,threadtable_size * sizeof(thread_act_t));
+                                if (task_threads(task, &threadtable, &threadtable_size) == KERN_SUCCESS) {
+                                        threadinfo = &threadinfo_data;
+                                        for (int j = 0; j < threadtable_size; j++) {
+                                                count = THREAD_BASIC_INFO_COUNT;
+                                                if (thread_info(threadtable[j], THREAD_BASIC_INFO, (thread_info_t)threadinfo, &count) == KERN_SUCCESS) {
+                                                        if ((threadinfo->flags & TH_FLAGS_IDLE) == 0) {
+                                                                pt[i].cputime += (long)((threadinfo->user_time.seconds + threadinfo->system_time.seconds) * 10 + (threadinfo->user_time.microseconds + threadinfo->system_time.microseconds) / 100000);
+                                                                pt[i].cpu_percent = 0;
+                                                        }
+                                                }
+                                                mach_port_deallocate(mytask, threadtable[j]);
+                                        }
+                                        vm_deallocate(mytask, (vm_address_t)threadtable,threadtable_size * sizeof(thread_act_t));
+                                }
+                                mach_port_deallocate(mytask, task);
                         }
-                        mach_port_deallocate(mytask, task);
                 }
         }
         FREE(args);
         FREE(pinfo);
-
+        
         *reference = pt;
-
+        
         return (int)treesize;
 }
 
@@ -285,30 +342,20 @@ int getloadavg_sysdep (double *loadv, int nelem) {
  * @return: true if successful, false if failed (or not available)
  */
 boolean_t used_system_memory_sysdep(SystemInfo_T *si) {
-        int                    mib[2];
-        unsigned int           pw, pa, pi;
-        size_t                 len;
-        struct xsw_usage       swap;
-        kern_return_t          kret;
-        vm_statistics_data_t   page_info;
-        mach_msg_type_number_t count;
-
         /* Memory */
-        count = HOST_VM_INFO_COUNT;
-        kret  = host_statistics(mach_host_self(), HOST_VM_INFO, (host_info_t)&page_info, &count);
+        vm_statistics_data_t page_info;
+        mach_msg_type_number_t count = HOST_VM_INFO_COUNT;
+        kern_return_t kret = host_statistics(mach_host_self(), HOST_VM_INFO, (host_info_t)&page_info, &count);
         if (kret != KERN_SUCCESS) {
                 DEBUG("system statistic error -- cannot get memory usage\n");
                 return false;
         }
-        pw = page_info.wire_count * pagesize_kbyte;
-        pa = page_info.active_count * pagesize_kbyte;
-        pi = page_info.inactive_count * pagesize_kbyte;
-        si->total_mem_kbyte = pw + pa + pi;
+        si->total_mem_kbyte = (page_info.wire_count + page_info.active_count) * pagesize_kbyte;
 
         /* Swap */
-        mib[0] = CTL_VM;
-        mib[1] = VM_SWAPUSAGE;
-        len    = sizeof(struct xsw_usage);
+        int mib[2] = {CTL_VM, VM_SWAPUSAGE};
+        size_t len = sizeof(struct xsw_usage);
+        struct xsw_usage swap;
         if (sysctl(mib, 2, &swap, &len, NULL, 0) == -1) {
                 DEBUG("system statistic error -- cannot get swap usage: %s\n", STRERROR);
                 si->swap_kbyte_max = 0;
@@ -351,4 +398,3 @@ boolean_t used_system_cpu_sysdep(SystemInfo_T *si) {
         }
         return false;
 }
-
