@@ -213,10 +213,16 @@ boolean_t sendmail(Mail_T mail) {
                 if (S.ssl.use_ssl && (S.ssl.version == SSL_TLSV1 || S.ssl.version == SSL_TLSV11 || S.ssl.version == SSL_TLSV12)) {
                         do_send(&S, "STARTTLS\r\n");
                         do_status(&S);
-                        if (! Socket_enableSsl(S.socket, S.ssl, NULL)) {
-                                S.quit = false;
-                                THROW(IOException, "Cannot switch to SSL");
+                        TRY
+                        {
+                                Socket_enableSsl(S.socket, S.ssl, NULL);
                         }
+                        ELSE
+                        {
+                                S.quit = false;
+                                RETHROW;
+                        }
+                        END_TRY;
                         /* After starttls, send ehlo again: RFC 3207: 4.2 Result of the STARTTLS Command */
                         do_send(&S, "EHLO %s\r\n", S.localhost);
                         do_status(&S);
