@@ -242,9 +242,17 @@ T _createIpSocket(const char *host, const struct sockaddr *addr, socklen_t addrl
                                         S->host = Str_dup(host);
                                         S->port = _getPort(addr, addrlen);
                                         S->connection_type = Connection_Client;
-                                        if (ssl.use_ssl && ! Socket_enableSsl(S, ssl, host)) {
-                                                Socket_free(&S);
-                                                THROW(IOException, "Could not switch socket to SSL");
+                                        if (ssl.use_ssl) {
+                                                TRY
+                                                {
+                                                        Socket_enableSsl(S, ssl, host);
+                                                }
+                                                ELSE
+                                                {
+                                                        Socket_free(&S);
+                                                        RETHROW;
+                                                }
+                                                END_TRY;
                                         }
                                         return S;
                                 }
@@ -344,8 +352,8 @@ T Socket_create(const char *host, int port, Socket_Type type, Socket_Family fami
         ASSERT(timeout > 0);
         volatile T S = NULL;
         struct addrinfo *result = _resolve(host, port, type, family);
-        char error[STRLEN];
         if (result) {
+                char error[STRLEN];
                 // The host may resolve to multiple IPs and if at least one succeeded, we have no problem and don't have to flood the log with partial errors => log only the last error
                 for (struct addrinfo *r = result; r && S == NULL; r = r->ai_next) {
                         TRY
@@ -359,9 +367,9 @@ T Socket_create(const char *host, int port, Socket_Type type, Socket_Family fami
                         END_TRY;
                 }
                 freeaddrinfo(result);
+                if (! S)
+                        LogError("Cannot create socket to [%s]:%d -- %s\n", host, port, error);
         }
-        if (! S)
-                LogError("Cannot create socket to [%s]:%d -- %s\n", host, port, error);
         return S;
 }
 
@@ -390,7 +398,7 @@ T Socket_createAccepted(int socket, struct sockaddr *addr, socklen_t addrlen, vo
 #ifdef HAVE_OPENSSL
                 if (sslserver) {
                         S->sslserver = sslserver;
-                        if (! (S->ssl = SslServer_newConnection(S->sslserver)) || ! SslServer_accept(S->ssl, S->socket)) {
+                        if (! (S->ssl = SslServer_newConnection(S->sslserver)) || ! SslServer_accept(S->ssl, S->socket, S->timeout)) {
                                 Socket_free(&S);
                                 return NULL;
                         }
@@ -510,15 +518,12 @@ const char *Socket_getLocalHost(T S, char *host, int hostlen) {
 
 
 static void _testUnix(Port_T p) {
-        long long start = Time_milli();
-        T S = _createUnixSocket(p->pathname, p->type, p->timeout);
+        T S = _createUnixSocket(p->target.unix.pathname, p->type, p->timeout);
         if (S) {
                 S->Port = p;
                 TRY
                 {
                         p->protocol->check(S);
-                        p->is_available = true;
-                        p->response = (Time_milli() - start) / 1000.;
                 }
                 FINALLY
                 {
@@ -526,26 +531,25 @@ static void _testUnix(Port_T p) {
                 }
                 END_TRY;
         } else {
-                THROW(IOException, "Cannot create unix socket for %s", p->pathname);
+                THROW(IOException, "Cannot create unix socket for %s", p->target.unix.pathname);
         }
 }
 
 
 static void _testIp(Port_T p) {
         char error[STRLEN];
-        struct addrinfo *result = _resolve(p->hostname, p->port, p->type, p->family);
+        boolean_t is_available = false;
+        struct addrinfo *result = _resolve(p->hostname, p->target.net.port, p->type, p->family);
         if (result) {
                 // The host may resolve to multiple IPs and if at least one succeeded, we have no problem and don't have to flood the log with partial errors => log only the last error
-                for (struct addrinfo *r = result; r && ! p->is_available; r = r->ai_next) {
+                for (struct addrinfo *r = result; r && ! is_available; r = r->ai_next) {
                         volatile T S = NULL;
                         TRY
                         {
-                                long long start = Time_milli();
-                                S = _createIpSocket(p->hostname, r->ai_addr, r->ai_addrlen, r->ai_family, r->ai_socktype, r->ai_protocol, p->SSL, p->timeout);
+                                S = _createIpSocket(p->hostname, r->ai_addr, r->ai_addrlen, r->ai_family, r->ai_socktype, r->ai_protocol, p->target.net.ssl, p->timeout);
                                 S->Port = p;
                                 p->protocol->check(S);
-                                p->is_available = true;
-                                p->response = (Time_milli() - start) / 1000.;
+                                is_available = true;
                         }
                         ELSE
                         {
@@ -560,10 +564,10 @@ static void _testIp(Port_T p) {
                         END_TRY;
                 }
                 freeaddrinfo(result);
-                if (! p->is_available)
+                if (! is_available)
                         THROW(IOException, "%s", error);
         } else {
-                THROW(IOException, "Cannot resolve [%s]:%d", p->hostname, p->port);
+                THROW(IOException, "Cannot resolve [%s]:%d", p->hostname, p->target.net.port);
         }
 }
 
@@ -574,31 +578,68 @@ static void _testIp(Port_T p) {
 void Socket_test(void *P) {
         ASSERT(P);
         Port_T p = P;
-        p->response = -1;
-        p->is_available = false;
-        switch (p->family) {
-                case Socket_Unix:
-                        _testUnix(p);
-                        break;
-                case Socket_Ip:
-                case Socket_Ip4:
-                case Socket_Ip6:
-                        _testIp(p);
-                        break;
-                default:
-                        LogError("Invalid socket family %d\n", p->family);
-                        break;
+        TRY
+        {
+                long long start = Time_milli();
+                switch (p->family) {
+                        case Socket_Unix:
+                                _testUnix(p);
+                                break;
+                        case Socket_Ip:
+                        case Socket_Ip4:
+                        case Socket_Ip6:
+                                _testIp(p);
+                                break;
+                        default:
+                                THROW(IOException, "Invalid socket family %d\n", p->family);
+                                break;
+                }
+                p->is_available = true;
+                p->response = (Time_milli() - start) / 1000.;
         }
+        ELSE
+        {
+                p->is_available = false;
+                p->response = -1;
+                RETHROW;
+        }
+        END_TRY;
 }
 
 
-boolean_t Socket_enableSsl(T S, SslOptions_T ssl, const char *name)  {
+void Socket_enableSsl(T S, SslOptions_T ssl, const char *name)  {
         assert(S);
 #ifdef HAVE_OPENSSL
-        if ((S->ssl = Ssl_new(ssl.clientpemfile, ssl.version)) && Ssl_connect(S->ssl, S->socket, name) && (! ssl.certmd5 || Ssl_checkCertificate(S->ssl, ssl.certmd5)))
-                return true;
+        if ((S->ssl = Ssl_new(ssl.version != -1 ? ssl.version : Run.ssl.version != -1 ? Run.ssl.version : SSL_Auto,
+                              ssl.CACertificateFile ? ssl.CACertificateFile : Run.ssl.CACertificateFile ? Run.ssl.CACertificateFile : NULL,
+                              ssl.CACertificatePath ? ssl.CACertificatePath : Run.ssl.CACertificatePath ? Run.ssl.CACertificatePath : NULL,
+                              ssl.clientpemfile ? ssl.clientpemfile : Run.ssl.clientpemfile ? Run.ssl.clientpemfile : NULL)))
+        {
+                // Set SSL options with fallback to global SSL options
+
+                if (ssl.verify != -1)
+                        Ssl_setVerifyCertificates(S->ssl, ssl.verify);
+                else if (Run.ssl.verify != -1)
+                        Ssl_setVerifyCertificates(S->ssl, Run.ssl.verify);
+
+                if (ssl.allowSelfSigned != -1)
+                        Ssl_setAllowSelfSignedCertificates(S->ssl, ssl.allowSelfSigned);
+                else if (Run.ssl.allowSelfSigned != -1)
+                        Ssl_setAllowSelfSignedCertificates(S->ssl, Run.ssl.allowSelfSigned);
+
+                if (ssl.minimumValidDays > 0)
+                        Ssl_setCertificateMinimumValidDays(S->ssl, ssl.minimumValidDays);
+                else if (Run.ssl.minimumValidDays > 0)
+                        Ssl_setCertificateMinimumValidDays(S->ssl, Run.ssl.minimumValidDays);
+
+                if (ssl.checksum)
+                        Ssl_setCertificateChecksum(S->ssl, ssl.checksumType, ssl.checksum);
+                else if (Run.ssl.checksum)
+                        Ssl_setCertificateChecksum(S->ssl, Run.ssl.checksumType, Run.ssl.checksum);
+
+                Ssl_connect(S->ssl, S->socket, S->timeout, name);
+        }
 #endif
-        return false;
 }
 
 
